#ifndef __myavr_h__
#define __myavr_h__

#ifndef sbi
	#define sbi(port, bit) (port) |= (1 << (bit))
#endif
#ifndef cbi
	#define cbi(port, bit) (port) &= ~(1 << (bit))
#endif

#ifndef BIT0
	#define BIT0 1
#endif
#ifndef BIT1
	#define BIT1 2
#endif
#ifndef BIT2
	#define BIT2 4
#endif
#ifndef BIT3
	#define BIT3 8
#endif
#ifndef BIT4
	#define BIT4 16
#endif
#ifndef BIT5
	#define BIT5 32
#endif
#ifndef BIT6
	#define BIT6 64
#endif
#ifndef BIT7
	#define BIT7 128
#endif

typedef uint8_t  byte;
typedef uint8_t  U8;
typedef uint16_t U16;


//-------------------------------------------------------------------------
// Verwendung der Makros:
// Deklaration  der Bits z.B.:
//     #define LEDrot myPin(PORTB,2)
//     #define PumpeKH myPin(PORTB,1)
// Schalten der Bits z.B.:
//     on(LEDrot);
//     off(LEDrot);
//-------------------------------------------------------------------------
#ifndef outb
#define outb(xPORT,xBYTE) \
	{{	\
		asm volatile ("st Z, %1" : : "z" ((uint8_t)0xff&(uint16_t)(&xPORT)), "a" (xBYTE) );	\
	}}
#endif
#define SetPins(xPORT,xBITS) \
	{{	\
		asm volatile ("ld r16,Z" "\n\t"	"or r16,%1" "\n\t" "st Z, r16" : : "z" ((uint8_t)0xff&(uint16_t)(&xPORT)), "a" (xBITS) : "r16");	\
	}}
#define ClearPins(xPORT,xBITS) \
	{{	\
		asm volatile ("ld r16,Z" "\n\t"	"and r16,%1" "\n\t" "st Z, r16" : : "z" ((uint8_t)0xff&(uint16_t)(&xPORT)), "a" (~xBITS) : "r16");	\
	}}
#define _setPins(xPORT,xBITS) \
	{{	\
		asm volatile ("ld r16,Z" "\n\t"	"or r16,%1" "\n\t" "st Z, r16" : : "z" (xPORT), "a" (xBITS) : "r16");	\
	}}
#define _clrPins(xPORT,xBITS) \
	{{	\
		asm volatile ("ld r16,Z" "\n\t"	"and r16,%1" "\n\t" "st Z, r16" : : "z" (xPORT), "a" (~xBITS) : "r16");	\
	}}

#define MyPinOn(xMyPin) _setPins(xMyPin&0xff,xMyPin/0x100)
#define MyPinOff(xMyPin) _clrPins(xMyPin&0xff,(xMyPin/0x100))
#define MyPin(xPORT,xBIT) (((uint8_t)0xff&(uint16_t)(&xPORT))+(1<<xBIT)*0x100)


#ifdef F_CPU
#define waitMs(milliSec) \
{{ \
	__asm__ __volatile__ ( \
		"mov r16,%2" "\n\t" \
		"mov r17,%3" "\n\t" \
		"waitMs1%=:" "\n\t" \
		"movw %1,r16" "\n\t" \
		"waitMs2%=: sbiw %1,1" "\n\t" \
		"brne waitMs2%=" "\n\t" \
		"sbiw %[ms],1" "\n\t" \
		"brne waitMs1%=" "\n\t" \
		: \
		: [ms] "w" (milliSec),"w" (0),"r" (0xFF & (F_CPU/(1000*4))), "r" ((F_CPU/(1000*4))/0x100)  \
		: "r16","r17"\
	); \
}}
#else
	#warning F_CPU nicht definiert
#endif
///////////////// RESET /////////////
#define MyRESET \
{{ \
	__asm__ __volatile__ ( \
		"cli" "\n\t" \
		"ldi r30,0" "\n\t" \
		"ldi r31,0" "\n\t" \
		"ijmp" "\n\t" \
		: \
		: \
		: \
	); \
}}

//////// UART-Dynamische Registerdefinition ////////////////////////////
#define MyUDR (*(port))
#define MyUBRRH (*(port-1))
#define MyUBRRL (*(port-2))
#define MyUCSRC (*(port-4))
#define MyUCSRB (*(port-5))
#define MyUCSRA (*(port-6))
//////// Timer-Dynamische Registerdefinition ////////////////////////////
#define My8TCCRnA (*(port))
#define My8TCCRnB (*(port+1))
#define My8TCNTn  (*(port+2))
#define My8OCRnA  (*(port+3))
#define My8OCRnB  (*(port+4))

#define My16TCCRnA (*(port))
#define My16TCCRnB (*(port+1))
#define My16TCCRnC (*(port+2))
#define My16TCNTn  (*(volatile uint16_t*)(port+4))
#define My16TCNTnL (*(port+4))
#define My16TCNTnH (*(port+5))
#define My16ICRn   (*(volatile uint16_t*)(port+6))
#define My16ICRnL  (*(port+6))
#define My16ICRnH  (*(port+7))
#define My16OCRnA  (*(volatile uint16_t*)(port+8))
#define My16OCRnAL (*(port+8))
#define My16OCRnAH (*(port+9))
#define My16OCRnB  (*(volatile uint16_t*)(port+10))
#define My16OCRnBL (*(port+10))
#define My16OCRnBH (*(port+11))
#define My16OCRnC  (*(volatile uint16_t*)(port+12))
#define My16OCRnCL (*(port+12))
#define My16OCRnCH (*(port+13))

//////// Debug-Definitionen, falls Debug nicht aktiviert ist
#define DebugPrintFlash(txt) ;
#define DebugPrintRam(txt,size) ; 
#define DebugPrintHex(txt,num) ; 
#define DebugPrintDez(txt,num) ; 
#define DebugPrintString(txt,string);
#define DebugPrintDump(pBuffer,anz) ;
#define DebugPrintAllocFreelist() ;
#ifdef IsDEBUG
#include "debugUart.h"
#endif

#endif // __myavr_h__
